@@ -2,9 +2,9 @@
 *            PMW error message handling          *
 *************************************************/
 
-/* Copyright Philip Hazel 2021 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: December 2020 */
-/* This file last modified: December 2023 */
+/* This file last modified: June 2025 */
 
 #include "pmw.h"
 
@@ -188,7 +188,7 @@ static error_struct error_data[] = {
 { ec_major,   "%s must be on the first note of a chord" },
 { ec_major,   "missing backslash after note options - skipping to end of bar or newline" },
 { ec_minor,   "tie or glissando precedes %s (will carry over to next note)" },
-{ ec_major,   "can't have both staccato and staccatissimo" },
+{ ec_major,   "SPARE ERROR" },
 /* 120-124 */
 { ec_major,   "conflicting stem direction requests %s" },
 { ec_major,   "the \\sw\\ option is only available for beamed notes when the\n"
@@ -219,7 +219,7 @@ static error_struct error_data[] = {
               "was not a multiple of 4 points (scaled to the size of the staves)\n" },
 { ec_failed,  "malformed page list on command line" },
 { ec_failed,  "page range out of order on command line" },
-{ ec_failed,  "internal error: ps_ffprintf() called with invalid % escape" },
+{ ec_failed,  "internal error: SFF() or SFD() called with invalid %% escape" },
 /* 140-144 */
 { ec_failed,  "bad -printscale value on command line" },
 { ec_failed,  "-printside must specify 1 or 2 on command line" },
@@ -255,7 +255,7 @@ aborts the run after outputting additional information. */
 /* 160-164 */
 { ec_major,   "there are no bars in movement %d: no MIDI data written" },
 { ec_major,   "incomplete irregular note group at end of bar" },
-{ ec_failed,  "malformed -dbd option data" },
+{ ec_failed,  "malformed -dbd option data (needs 1-3 comma-separated numbers)" },
 { ec_failed,  "-dbd error: %s %d does not exist" },
 { ec_warning, "there is underlay or overlay text left over at the end of stave %d" },
 /* 165-169 */
@@ -275,7 +275,23 @@ aborts the run after outputting additional information. */
 { ec_major,   "duplicate glyph name \"%s\" in line %d of %s\n%s" },
 { ec_major,   "duplicate font encoding value %d in line %d of %s\n%s" },
 { ec_major,   "[backup] must follow a note" },
-{ ec_warning, "non-movement options on rehearsal marks are ignored" }
+{ ec_warning, "non-movement options on rehearsal marks are ignored" },
+{ ec_major,   "'&' at end of line while reading macro or repetition argument" },
+/* 180-184 */
+{ ec_failed,  "repetition count is too large (max %d)" },
+{ ec_failed,  "-%s and -pdf are mutually exclusive" },
+{ ec_warning, "-%s is ignored with -pdf" },
+{ ec_failed,  "internal error: invalid output layout for PDF" },
+{ ec_warning, "could not find font \"%s\" (use -F option?)" },
+/* 185-189 */
+{ ec_failed,  "PMW in PDF mode does not support %s fonts (%s)" },
+{ ec_failed,  "File %s is not a recognized font file" },
+{ ec_failed,  "Type 3 font (%s) is not supported in PDF mode" },
+{ ec_failed,  "the \"%s\" directive is incompatible with a previous %s\n"
+              "   setting (either a directive or a command line option)" },
+{ ec_major,   "movement %d does not exist: no MIDI data written" },
+/* 190-194 */
+{ ec_major,   "misplaced [tremolo]: %s" }
 };
 
 #define ERROR_MAXERROR (int)(sizeof(error_data)/sizeof(error_struct))
@@ -374,32 +390,31 @@ if (main_state == STATE_READ)
 
   (void)fprintf(stderr, "%s", buffer);
 
-  /* Show where in the line, except at end of file. */
-
-  if (read_c != ENDFILE)
-    {
-    for (usint i = 0; i < in - 1; i++) (void)fprintf(stderr, "-");
-    (void)fprintf(stderr, ">\n");
-    }
-
-  /* For certain input errors we skip along the input to one or two designated
+  /* Unless we are at the end of the file, show where in the line we are. Then,
+  for certain input errors we skip along the input to one or two designated
   characters. In all cases, stop at the end of the line. The skip setting
   should never happen while expanding macros, but double-check just in case. */
 
-  if (!macro_expanding && skip != 0)
+  if (read_filehandle != NULL && in != 0)
     {
-    if (skip == '\n')   /* Can optimize newline (alone) case */
+    for (usint i = 0; i < in - 1; i++) (void)fprintf(stderr, "-");
+    (void)fprintf(stderr, ">\n");
+
+    if (!macro_expanding && skip != 0)
       {
-      read_c = '\n';
-      read_i = main_readlength;
-      }
-    else
-      {
-      uint32_t skip1 = skip & 0xff;
-      uint32_t skip2 = skip >> 8;
-      while (read_c != ENDFILE && read_c != skip1 &&
-             read_c != skip2 && read_c != '\n')
-         read_nextc();
+      if (skip == '\n')   /* Can optimize newline (alone) case */
+        {
+        read_c = '\n';
+        read_i = main_readlength;
+        }
+      else
+        {
+        int32_t skip1 = skip & 0xff;
+        int32_t skip2 = skip >> 8;
+        while (read_c != ENDFILE && read_c != skip1 &&
+               read_c != skip2 && read_c != '\n')
+           read_nextc();
+        }
       }
     }
   }
