@@ -2,9 +2,11 @@
 *       PMW prototypes for global functions      *
 *************************************************/
 
-/* Copyright Philip Hazel 2022 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: December 2020 */
-/* This file last modified: December 2023 */
+/* This file last modified: April 2025 */
+
+/* -------- General functions -------- */
 
 extern void         debug_bar(void);
 extern void         debug_header(void);
@@ -19,7 +21,6 @@ extern int32_t      font_charwidth(uint32_t, uint32_t, uint32_t, int32_t,
                       int32_t *);
 extern FILE        *font_finddata(uschar *, const char *, uschar *, uschar *,
                       uschar *, BOOL);
-extern void         font_loadtables(uint32_t);
 extern uint32_t     font_readtype(BOOL);
 extern fontinststr *font_rotate(fontinststr *, int32_t);
 extern uint32_t     font_search(uschar *);
@@ -85,41 +86,28 @@ extern void         out_writetime(int32_t, int32_t, usint);
 
 extern void         paginate(void);
 
+extern void         pout_beam(int32_t *, int32_t *, int32_t *, int32_t *,
+                      int32_t *, int, int);
+extern void         pout_getcolour(int32_t *);
+extern int32_t      pout_getswidth(uint32_t *, usint, fontstr *, int32_t *,
+                      int32_t *);
+extern void         pout_muschar(int32_t, int32_t, uint32_t, int32_t,
+                      void(*)(uint32_t *, usint, fontinststr *, int32_t,
+                      int32_t));
+extern BOOL         pout_get_pages(pagestr **, pagestr **);
+extern void         pout_set_ymax_etc(int32_t *, int32_t *);
+extern void         pout_setcolour(int32_t *);
+extern void         pout_setgray(int32_t);
+extern void         pout_setup_pagelist(BOOL);
+extern void         pout_string(uint32_t *, fontinststr *, int32_t *,
+                      int32_t *, BOOL, void(*)(uint32_t *, usint, fontinststr *,
+                      int32_t, int32_t));
 extern void         pmw_read(void);
 extern void         pmw_read_header(void);
 extern void         pmw_read_stave(void);
 
 extern void         preprocess_line(void);
 
-extern void         ps_abspath(int32_t *, int32_t *, int *, int32_t);
-extern void         ps_barline(int32_t, int32_t, int32_t, int, int32_t);
-extern void         ps_beam(int32_t, int32_t, int, int);
-extern void         ps_brace(int32_t, int32_t, int32_t, int32_t);
-extern void         ps_bracket(int32_t, int32_t, int32_t, int32_t);
-extern void         ps_getcolour(int32_t *);
-extern void         ps_go(void);
-extern void         ps_grestore(void);
-extern void         ps_gsave(void);
-extern void         ps_line(int32_t, int32_t, int32_t, int32_t, int32_t,
-                      uint32_t);
-extern void         ps_lines(int32_t *, int32_t *, int, int32_t);
-extern void         ps_muschar(int32_t, int32_t, uint32_t, int32_t);
-extern void         ps_musstring(uschar *, int32_t, int32_t, int32_t);
-extern void         ps_path(int32_t *, int32_t *, int *, int32_t);
-extern void         ps_relmusstring(uschar *, int32_t, int32_t, int32_t);
-extern void         ps_rotate(double);
-extern void         ps_setcapandjoin(uint32_t);
-extern void         ps_setcolour(int32_t *);
-extern void         ps_setdash(int32_t, int32_t);
-extern void         ps_setgray(int32_t);
-extern void         ps_slur(int32_t, int32_t, int32_t, int32_t, uint32_t,
-                      int32_t);
-extern void         ps_startbar(int, int);
-extern void         ps_stave(int32_t, int32_t, int32_t, int);
-extern void         ps_string(uint32_t *, fontinststr *, int32_t *, int32_t *,
-                      BOOL, BOOL);
-extern void         ps_translate(int32_t, int32_t);
-
 extern uint32_t     read_accororn(uint32_t);
 extern uint32_t     read_barnumber(void);
 extern uint32_t     read_compute_barlength(uint32_t);
@@ -164,9 +152,11 @@ extern uint32_t    *string_check(uint32_t *, const char *, BOOL);
 extern int          string_check_utf8(uschar *);
 extern void         string_extend_buffer(void);
 extern char        *string_format_barnumber(uint32_t);
+extern char        *string_format_double(double);
 extern char        *string_format_fixed(int32_t);
 extern char        *string_format_key(uint32_t);
 extern char        *string_format_multiple_fixed(const char *, ...);
+extern char        *string_format_multiple_double(const char *, ...);
 extern char        *string_format_notelength(int32_t);
 extern char        *string_format_pitch(uint16_t, BOOL);
 extern BOOL         string_pmweq(uint32_t *, uint32_t *);
@@ -176,7 +166,7 @@ extern BOOL         string_read_plain(void);
 extern void         string_relativize(void);
 extern void         string_stavestring(BOOL);
 extern int32_t      string_width(uint32_t *, fontinststr *, int32_t *);
-extern int          strncmpic(const char*, const char *, int);
+extern int          strncmpic(const unsigned char*, const unsigned char *, int);
 
 extern uint32_t     transpose_key(uint32_t);
 extern int16_t      transpose_note(int16_t, int16_t *, uint8_t *, uint8_t,
@@ -189,4 +179,97 @@ extern tree_node   *tree_search(tree_node *, uschar *);
 extern void         xml_read(void);
 #endif
 
+
+/* -------- PostScript output functions -------- */
+
+extern void         ps_abspath(int32_t *, int32_t *, int *, int32_t);
+extern void         ps_barline(int32_t, int32_t, int32_t, int, int32_t);
+extern void         ps_beam(int32_t, int32_t, int, int);
+extern void         ps_brace(int32_t, int32_t, int32_t, int32_t);
+extern void         ps_bracket(int32_t, int32_t, int32_t, int32_t);
+extern void         ps_getcolour(int32_t *);
+extern void         ps_go(void);
+extern void         ps_grestore(void);
+extern void         ps_gsave(void);
+extern void         ps_line(int32_t, int32_t, int32_t, int32_t, int32_t,
+                      uint32_t);
+extern void         ps_lines(int32_t *, int32_t *, int, int32_t);
+extern void         ps_muschar(int32_t, int32_t, uint32_t, int32_t);
+extern void         ps_musstring(uschar *, int32_t, int32_t, int32_t);
+extern void         ps_path(int32_t *, int32_t *, int *, int32_t);
+extern void         ps_rotate(double);
+extern void         ps_setcapandjoin(uint32_t);
+extern void         ps_setcolour(int32_t *);
+extern void         ps_setdash(int32_t, int32_t);
+extern void         ps_setgray(int32_t);
+extern void         ps_slur(int32_t, int32_t, int32_t, int32_t, uint32_t,
+                      int32_t);
+extern void         ps_startbar(int, int);
+extern void         ps_stave(int32_t, int32_t, int32_t, int);
+extern void         ps_string(uint32_t *, fontinststr *, int32_t *, int32_t *,
+                      BOOL);
+extern void         ps_translate(int32_t, int32_t);
+
+
+/* -------- PDF output functions -------- */
+
+extern void         pdf_abspath(int32_t *, int32_t *, int *, int32_t);
+extern void         pdf_barline(int32_t, int32_t, int32_t, int, int32_t);
+extern void         pdf_beam(int32_t, int32_t, int, int);
+extern void         pdf_brace(int32_t, int32_t, int32_t, int32_t);
+extern void         pdf_bracket(int32_t, int32_t, int32_t, int32_t);
+extern void         pdf_free_data(void);
+extern void         pdf_getcolour(int32_t *);
+extern void         pdf_go(void);
+extern void         pdf_grestore(void);
+extern void         pdf_gsave(void);
+extern void         pdf_line(int32_t, int32_t, int32_t, int32_t, int32_t,
+                      uint32_t);
+extern void         pdf_lines(int32_t *, int32_t *, int, int32_t);
+extern void         pdf_muschar(int32_t, int32_t, uint32_t, int32_t);
+extern void         pdf_musstring(uschar *, int32_t, int32_t, int32_t);
+extern void         pdf_path(int32_t *, int32_t *, int *, int32_t);
+extern void         pdf_rotate(double);
+extern void         pdf_setcapandjoin(uint32_t);
+extern void         pdf_setcolour(int32_t *);
+extern void         pdf_setdash(int32_t, int32_t);
+extern void         pdf_setgray(int32_t);
+extern void         pdf_slur(int32_t, int32_t, int32_t, int32_t, uint32_t,
+                      int32_t);
+extern void         pdf_startbar(int, int);
+extern void         pdf_stave(int32_t, int32_t, int32_t, int);
+extern void         pdf_string(uint32_t *, fontinststr *, int32_t *, int32_t *,
+                      BOOL);
+extern void         pdf_translate(int32_t, int32_t);
+
+
+/* -------- Indirections for switching between PostScript and PDF -------- */
+
+extern void         (*ofi_abspath)(int32_t *, int32_t *, int *, int32_t);
+extern void         (*ofi_barline)(int32_t, int32_t, int32_t, int, int32_t);
+extern void         (*ofi_beam)(int32_t, int32_t, int, int);
+extern void         (*ofi_brace)(int32_t, int32_t, int32_t, int32_t);
+extern void         (*ofi_bracket)(int32_t, int32_t, int32_t, int32_t);
+extern void         (*ofi_getcolour)(int32_t *);
+extern void         (*ofi_grestore)(void);
+extern void         (*ofi_gsave)(void);
+extern void         (*ofi_line)(int32_t, int32_t, int32_t, int32_t, int32_t,
+                      uint32_t);
+extern void         (*ofi_lines)(int32_t *, int32_t *, int, int32_t);
+extern void         (*ofi_muschar)(int32_t, int32_t, uint32_t, int32_t);
+extern void         (*ofi_musstring)(uschar *, int32_t, int32_t, int32_t);
+extern void         (*ofi_path)(int32_t *, int32_t *, int *, int32_t);
+extern void         (*ofi_rotate)(double);
+extern void         (*ofi_setcapandjoin)(uint32_t);
+extern void         (*ofi_setcolour)(int32_t *);
+extern void         (*ofi_setdash)(int32_t, int32_t);
+extern void         (*ofi_setgray)(int32_t);
+extern void         (*ofi_slur)(int32_t, int32_t, int32_t, int32_t, uint32_t,
+                      int32_t);
+extern void         (*ofi_startbar)(int, int);
+extern void         (*ofi_stave)(int32_t, int32_t, int32_t, int);
+extern void         (*ofi_string)(uint32_t *, fontinststr *, int32_t *,
+                       int32_t *, BOOL);
+extern void         (*ofi_translate)(int32_t, int32_t);
+
 /* End of functions.h */
