@@ -2,9 +2,9 @@
 *        PMW entry point and initialization      *
 *************************************************/
 
-/* Copyright Philip Hazel 2022 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: December 2020 */
-/* This file last modified: June 2023 */
+/* This file last modified: June 2025 */
 
 #include "pmw.h"
 #include "rdargs.h"
@@ -45,6 +45,7 @@ static const char *arg_pattern =
   "H/k,"
   "-help=help/s,"
   "incPMWfont=includePMWfont=incpmwfont=includepmwfont=ipf/s,"
+  "MacOSapp/k,"
   "MF/k,"
   "MP/k,"
   "MV/k,"
@@ -58,15 +59,17 @@ static const char *arg_pattern =
   "o/k,"
   "p/k,"
   "pamphlet/s,"
+  "pdf/s,"
   "printadjust/k/2/m,"
   "printgutter/k,"
   "printscale/k,"
   "printside/k/n,"
+  "ps/s,"
   "reverse/s,"
   "SM/k,"
   "s/k,"
   "t/k/n,"
-  "testing/s,"
+  "testing/n=2,"
   "tumble/s,"
   "-version=V/s,"
   "v/s";
@@ -93,6 +96,7 @@ enum {
   arg_H,
   arg_help,
   arg_incPMWfont,
+  arg_MacOSapp,
   arg_MF,
   arg_MP,
   arg_MV,
@@ -106,11 +110,13 @@ enum {
   arg_o,
   arg_p,
   arg_pamphlet,
+  arg_pdf,
   arg_printadjustx,
   arg_printadjusty,
   arg_printgutter,
   arg_printscale,
   arg_printside,
+  arg_ps,
   arg_reverse,
   arg_SM,
   arg_s,
@@ -167,7 +173,7 @@ static debug_bit_table debug_options[] = {
 *************************************************/
 
 int
-strncmpic(const char *s, const char *t, int n)
+strncmpic(const unsigned char *s, const unsigned char *t, int n)
 {
 while (n--)
   {
@@ -208,7 +214,7 @@ for(;;)
 
   adding = *string++ == '+';
   s = string;
-  while (isalnum(*string) || *string == '_') string++;
+  while (isalnum((unsigned char)*string) || *string == '_') string++;
   len = string - s;
 
   start = debug_options;
@@ -456,68 +462,75 @@ pmwrc = "no";
 #endif
 
 (void)printf("PMW version %s\n%s\n\n", PMW_VERSION, COPYRIGHT);
+(void)printf("Default output is:    %s\n", PDF? "PDF" : "PostScript");
 (void)printf("B2PF support:         %s\n", b2pf);
 (void)printf("~/.pmwrc support:     %s\n", pmwrc);
 
-PF("\nOPTIONS\n\n");
+PF("\nDefault output is <input>.ps or <input>.pdf when an input file name is given.\n");
+PF("Default output is stdout if no input file name is given.\n");
+
+PF("\nGENERAL OPTIONS\n\n");
 PF("-a4ona3               print A4 images 2-up on A3\n");
 PF("-a5ona4               print A5 images 2-up on A4\n");
-PF("-a4sideways           assume A4 paper fed sideways\n");
 PF("-C <arg>              show a compile-time option; exit with its value (0 or 1).\n");
 PF("    b2pf              support for B2PF processing\n");
 PF("    musicxml          support for MusicXML input\n");
-PF("-c <number>           set number of copies\n");
-PF("-d<options>           write debugging info to stderr\n");
-PF("-dbd <m>,<s>,<b>      write debugging bar data (movement, stave, bar) \n");
-PF("-dbl                  synonym for -drawbarlines\n");
-PF("-drawbarlines         don't use characters for bar lines\n");
+PF("-drawbarlines or -dbl don't use characters for bar lines\n");
 PF("-drawstavelines [<n>] don't use characters for stave lines\n");
 PF("-dsl [<n>]            synonym for -drawstavelines\n");
-PF("-dtp <bar>            write debugging position data (-1 for all bars)\n");
-PF("-duplex               set duplex printing in the PostScript\n");
-PF("-em <n>               synonym for -errormaximum\n");
 PF("-eps                  output encapsulated PostScript\n");
-PF("-errormaximum <n>     set maximum number of errors (for testing)\n");
-PF("-F <directory-list>   specify fontmetrics and/or .utr directories\n");
+PF("-F <directory-list>   specify fontmetrics, .utr, and font directories\n");
 PF("-f <name>             specify format name\n");
-PF("-H <file>             specify PostScript header file\n");
-PF("-help                 output this information\n");
-PF("-incPMWfont           include PMW font in the output\n");
-PF("-ipf                  synonym for -incPMWfont\n");
-PF("-MF <directory-list>  specify PostScript music fonts directories\n");
+PF("-help                 output this information, then exit\n");
 PF("-MP <file>            specify MIDIperc file\n");
 PF("-MV <file>            specify MIDIvoices file\n");
-PF("-manualfeed           set manualfeed in the PostScript\n");
 PF("-mb <range>           synonym for -midibars\n");
 PF("-midi <file>          specify MIDI output file\n");
 PF("-midibars <range>     limit MIDI output to given bar range\n");
 PF("-midimovement <n>     specifies movement for MIDI output\n");
 PF("-mm <n>               synonym for -midimovement\n");
-PF("-noid                 don't show version number for error output\n");
 #if !defined NO_PMWRC || NO_PMWRC == 0
 PF("-norc or -nopmwrc     don't read .pmwrc (must be first option)\n");
 #endif
-PF("-norepeats            do not play repeats in MIDI output\n");
-PF("-nowidechars          don't use 100-point stave chars\n");
-PF("-nr                   synonym for -norepeats\n");
-PF("-nw                   synonym for -nowidechars\n");
+PF("-norepeats or -nr     do not play repeats in MIDI output\n");
+PF("-nowidechars or -nw   don't use 100-point stave chars\n");
 PF("-o <file>             specify output file ('-' for stdout)\n");
 PF("-p <list>             select pages\n");
 PF("-pamphlet             print pages in pamphlet order\n");
+PF("-pdf                  select PDF output\n");
 PF("-printadjust <x> <y>  move on page by (x,y)\n");
 PF("-printgutter <x>      move recto/verso pages by x/-x\n");
 PF("-printscale <n>       scale the image by n\n");
 PF("-printside <n>        print only odd or even sides\n");
+PF("-ps                   select PostScript output\n");
 PF("-reverse              output pages in reverse order\n");
-PF("-SM <directory>       specify standard macros directory\n");
 PF("-s <list>             select staves\n");
 PF("-t <number>           set transposition\n");
-PF("-testing              run in testing mode\n");
-PF("-tumble               set tumble for duplex printing\n");
-PF("-V                    output PMW version number\n");
+PF("-V                    output PMW version number, then exit\n");
 PF("-v                    output verification information\n");
 
-PF("\nDebug options (+ to add, - to subtract):");
+PF("\nPOSTSCRIPT-SPECIFIC OPTIONS\n\n");
+PF("-a4sideways           assume A4 paper fed sideways\n");
+PF("-c <number>           set number of copies\n");
+PF("-duplex               set duplex printing in the PostScript\n");
+PF("-H <file>             specify PostScript header file\n");
+PF("-incPMWfont or -ipf   include PMW font in the output\n");
+PF("-manualfeed           set manualfeed in the PostScript\n");
+PF("-tumble               set tumble for duplex printing\n");
+
+PF("\nMAINTENANCE AND DEBUGGING OPTIONS\n\n");
+PF("-d<selectors>         write debugging info to stderr (see below)\n");
+PF("-dbd <m>,<s>,<b>      write debugging bar data (movement, stave, bar) \n");
+PF("-dtp <bar>            write debugging position data (-1 for all bars)\n");
+PF("-em <n>               synonym for -errormaximum\n");
+PF("-errormaximum <n>     set maximum number of errors (for testing)\n");
+PF("-MacOSapp <directory> resource directory when run from a MacOS app\n");
+PF("-MF <directory-list>  specify music font directories\n");
+PF("-noid                 don't show version number for error output\n");
+PF("-SM <directory>       specify standard macros directory\n");
+PF("-testing [<n>]        run in testing mode\n");
+
+PF("\nDebug selectors (+ to add, - to subtract):");
 for (long unsigned int i = 0; i < DEBUG_OPTIONS_COUNT; i++)
   {
   if ((i & 7) == 0) PF("\n ");
@@ -525,15 +538,11 @@ for (long unsigned int i = 0; i < DEBUG_OPTIONS_COUNT; i++)
   }
 PF("\n");
 
-PF("\nDefault output is <input>.ps when a file name is given.\n");
-PF("Default output is stdout if no file name is given.\n");
-
 PF("\nEXAMPLES\n\n");
 PF("pmw myscore\n");
 PF("pmw -s 1,2-4 -p 3,6-10,11 -f small -c 2 k491.pmw\n");
-PF("pmw -pamphlet -a5ona4 scorefile\n");
+PF("pmw -pdf -pamphlet -a5ona4 scorefile\n");
 PF("pmw -s 1 -midi zz.mid -mm 2 -mb 10-20 sonata\n");
-
 }
 
 
@@ -607,6 +616,31 @@ if (results[arg_help].number != 0)
   exit(EXIT_SUCCESS);
   }
 
+/* When PMW is being run as part of a MacOS app, the default locations for its
+resources are held within the app instead of being in (e.g.) /usr/local/share.
+The path to the relevant directory is passed via the -MacOSapp option. We
+adjust all the defaults here. If the app sets any overrides, they will be
+honoured. */
+
+if (results[arg_MacOSapp].text != NULL)
+  {
+  uschar *resources = US results[arg_MacOSapp].text;
+  size_t rlen = Ustrlen(resources);
+
+  font_data_default = resources;
+  font_music_default = resources;
+  stdmacs_dir = resources;
+
+  midi_perc = mem_get(rlen + 9);
+  sprintf(CS midi_perc, "%s/MIDIperc", resources);
+
+  midi_voices = mem_get(rlen + 11);
+  sprintf(CS midi_voices, "%s/MIDIvoices", resources);
+
+  ps_header = mem_get(rlen + 9);
+  sprintf(CS ps_header, "%s/PSheader", resources);
+  }
+
 /* Deal with verifying and debugging */
 
 if (results[arg_v].number != 0)
@@ -616,11 +650,8 @@ if (results[arg_v].number != 0)
   main_verify = TRUE;
   }
 
-if (results[arg_testing].number != 0)
-  {
-  main_testing = TRUE;
+if ((main_testing = results[arg_testing].number) != 0)
   main_showid = FALSE;
-  }
 
 if (results[arg_d].text != NULL)
   {
@@ -630,10 +661,11 @@ if (results[arg_d].text != NULL)
 
 if (results[arg_dbd].text != NULL)
   {
+  char trail[8];
   if (strspn(results[arg_dbd].text, "0123456789,") !=
-      strlen(results[arg_dbd].text)) error(ERR77);   /* Hard */
-  switch (sscanf(results[arg_dbd].text, "%d,%d,%d", &dbd_movement,
-          &dbd_stave, &dbd_bar))
+      strlen(results[arg_dbd].text)) error(ERR162);   /* Hard */
+  switch (sscanf(results[arg_dbd].text, "%d,%d,%d%1s", &dbd_movement,
+          &dbd_stave, &dbd_bar, trail))
     {
     case 1:                           /* One value is a bar number */
     dbd_bar = dbd_movement;
@@ -668,7 +700,6 @@ if (results[arg_aa_input].text != NULL)
 if (results[arg_o].text != NULL)
   out_filename = US results[arg_o].text;
 
-
 /* Deal with overriding music fonts, fontmetrics, and psheader, MIDIperc,
 MIDIvoices, and StdMacs files */
 
@@ -772,7 +803,10 @@ if (results[arg_t].presence != arg_present_not)
 /* Deal with format */
 
 if (results[arg_f].text != NULL)
+  {
   main_format = mem_copystring(US results[arg_f].text);
+  for (uschar *s = main_format; *s != 0; s++) *s = tolower(*s);
+  }
 
 /* Deal with copies */
 
@@ -786,12 +820,16 @@ if (results[arg_a4sideways].number != 0) print_pagefeed = pc_a4sideways;
 if (results[arg_a4ona3].number != 0) print_imposition = pc_a4ona3;
 if (results[arg_a5ona4].number != 0) print_imposition = pc_a5ona4;
 if (results[arg_pamphlet].number != 0) print_pamphlet = TRUE;
-if (results[arg_eps].number != 0) print_imposition = pc_EPS;
 if (results[arg_incPMWfont].number != 0) print_incPMWfont = TRUE;
 if (results[arg_manualfeed].number != 0) print_manualfeed = TRUE;
 if (results[arg_duplex].number != 0) print_duplex = TRUE;
 if (results[arg_tumble].number != 0) print_tumble = TRUE;
 
+if (results[arg_eps].number != 0)
+  {
+  print_imposition = pc_EPS;
+  EPSforced = TRUE;
+  }
 if (results[arg_printadjustx].text != NULL)
   {
   float d;
@@ -828,6 +866,40 @@ if (results[arg_printside].presence != arg_present_not)
     else if (n == 2) print_side1 = FALSE;
       else error(ERR141);  /* Hard */
   }
+
+/* Only one of -ps or -pdf is allowed; default is set at build time. We need to
+remember if the format is forced in order to handle the header directives that
+are equivalent to the command line options. */
+
+if (results[arg_pdf].number != 0)
+  {
+  if (results[arg_ps].number != 0) error(ERR181, "ps");  /* Hard */
+  PDF = PDFforced = TRUE;
+  }
+else if (results[arg_ps].number != 0 || print_imposition == pc_EPS)
+  {
+  PDF = FALSE;
+  PSforced = TRUE;
+  }
+
+/* Many PostScript-specific args are either ignored or cause a fatal error when
+output is PDF. */
+
+if (PDF)
+  {
+  if (print_imposition == pc_EPS) error(ERR181, "eps");  /* Hard */
+  if (print_copies != 1) error(ERR182, "c"); /* Warning */
+  if (print_pagefeed == pc_a4sideways)
+    {
+    error(ERR182, "a4sideways");                        /* Warning */
+    print_pagefeed = pc_normal;
+    }
+  if (print_duplex) error(ERR182, "duplex");            /* Warning */
+  if (print_tumble) error(ERR182, "tumble");            /* Warning */
+  if (print_manualfeed) error(ERR182, "manualfeed");    /* Warning */
+  if (results[arg_H].text != NULL) error(ERR182, "H");  /* Warning */
+  if (print_incPMWfont) error(ERR182, "incPMWfont");    /* Warning */
+  }
 }
 
 
@@ -910,7 +982,7 @@ else
       possibly doing damage. */
 
       if (rdargs(nargc, nargv, arg_pattern, results, MAX_COMMANDARGS) != 0)
-      error(ERR26, results[0].text, results[1].text);  /* Hard */
+        error(ERR26, results[0].text, results[1].text);  /* Hard */
       }
 
     /* stat() problem other than file not found is serious */
@@ -1019,16 +1091,13 @@ Returns:   nothing
 static void
 tidy_up(void)
 {
-usint i;
-
 if (read_filehandle != NULL) fclose(read_filehandle);
-
 free(font_list);
 
 #if defined SUPPORT_B2PF && SUPPORT_B2PF != 0
 if (font_b2pf_contexts != NULL)
   {
-  for (i = 0; i < font_tablen; i++)
+  for (usint i = 0; i < font_tablen; i++)
     if (font_b2pf_contexts[i] != NULL) b2pf_context_free(font_b2pf_contexts[i]);
   }
 #endif
@@ -1041,12 +1110,12 @@ free(read_stringbuffer);
 
 /* A NULL pointer marks the end of the macro argument expansion buffers. */
 
-for (i = 0; i < MAX_MACRODEPTH; i++)
+for (usint i = 0; i < MAX_MACRODEPTH; i++)
   if (main_argbuffer[i] != NULL) free(main_argbuffer[i]); else break;
 
 /* Free expandable vectors in all movements, then the movements vector. */
 
-for (i = 0; i < movement_count; i++)
+for (usint i = 0; i < movement_count; i++)
   {
   curmovt = movements[i];
   free(curmovt->barvector);
@@ -1054,7 +1123,6 @@ for (i = 0; i < movement_count; i++)
   for (int j = 0; j <= curmovt->lastreadstave; j++)
     if (curmovt->stavetable[j] != NULL) free(curmovt->stavetable[j]->barindex);
   }
-
 free(movements);
 
 /* An expandable XML buffer */
@@ -1063,6 +1131,10 @@ free(movements);
 free(xml_layout_list);
 #endif
 
+/* PDF expandable data areas */
+
+pdf_free_data();
+
 /* Free the non-expandable memory blocks */
 
 mem_free();
@@ -1105,12 +1177,12 @@ if (main_filename != NULL)
     {
     uschar *p, *q;
     size_t len = Ustrlen(main_filename);
-    out_filename = mem_get(len + 4);
+    out_filename = mem_get(len + 5);
     Ustrcpy(out_filename, main_filename);
     if ((p = Ustrrchr(out_filename, '.')) != NULL &&
         ((q = Ustrrchr(out_filename, '/')) == NULL || q < p))
       len = p - out_filename;
-    Ustrcpy(out_filename + len, ".ps");
+    Ustrcpy(out_filename + len, PDF? ".pdf" : ".ps");
     }
   }
 
@@ -1123,7 +1195,7 @@ else
 /* Read the input file */
 
 main_state = STATE_READ;
-if (main_verify) eprintf( "Reading...\n");
+if (main_verify) eprintf( "Reading input file\n");
 read_file(FT_AUTO);
 main_truepagelength = main_pagelength;  /* Save unscaled value */
 
@@ -1142,7 +1214,7 @@ if (!main_suppress_output)
   {
   main_state = STATE_PAGINATE;
   wk_cont = mem_get_independent((main_maxstave+1)*sizeof(contstr));
-  if (main_verify) eprintf( "Paginating...\n");
+  if (main_verify) eprintf( "Paginating\n");
   paginate();
   }
 
@@ -1163,14 +1235,15 @@ we'll be writing to stdout. */
 
 if (out_filename != NULL && Ustrcmp(out_filename, "-") != 0)
   {
-  if (main_verify) eprintf( "\nWriting PostScript file \"%s\"...\n", out_filename);
-  ps_file = Ufopen(out_filename, "w");
-  if (ps_file == NULL)
+  if (main_verify) eprintf( "\nWriting %s file \"%s\"\n",
+    PDF? "PDF" : "PostScript", out_filename);
+  out_file = Ufopen(out_filename, "w");
+  if (out_file == NULL)
     error(ERR23, out_filename, strerror(errno));  /* Hard error */
   }
 else
   {
-  ps_file = stdout;
+  out_file = stdout;
   if (main_verify) eprintf( "\nWriting Postscript to stdout...\n");
   }
 
@@ -1199,8 +1272,8 @@ print_pageorigin =
   (main_landscape? 4 : 0);
 
 main_state = STATE_WRITE;
-ps_go();
-if (ps_file != stdout) fclose(ps_file);
+if (PDF) pdf_go(); else ps_go();
+if (out_file != stdout) fclose(out_file);
 main_state = STATE_ENDING;
 
 /* Output warning if coupled staves were not spaced by a multiple of 4 */
