@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: August 2021 */
-/* This file last modified: July 2023 */
+/* This file last modified: June 2025 */
 
 #include "pmw.h"
 
@@ -310,14 +310,14 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
   BOOL noteson;   /* See above comment */
   int32_t moff;
   int midi_stave_status, midi_stave_pitch, midi_stave_velocity;
-  int miditranspose, adjustlength, tremolo;
+  int miditranspose, adjustlength, scrubtremolo;
 
   if (mac_notbit(midi_staves, stave)) continue;
 
   moff = 0;
   miditranspose = midi_transpose[stave];
   adjustlength = 0;
-  tremolo = -1;
+  scrubtremolo = -1;
   noteson = TRUE;
 
   /* Set up midi parameters */
@@ -396,10 +396,11 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
               if (pp->type == b_nbar)
                 {
                 second = i;
-                break;
+                goto FOUNDSECOND;
                 }
               }
             }
+          FOUNDSECOND:
 
           if (second > 0)
             {
@@ -474,8 +475,9 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
         {
         b_ornamentstr *orn = (b_ornamentstr *)p;
         if (orn->ornament == or_trem1 ||
-            orn->ornament == or_trem2)
-          tremolo = orn->ornament;
+            orn->ornament == or_trem2 ||
+            orn->ornament == or_trem3)
+          scrubtremolo = orn->ornament;
         }
       break;
 
@@ -487,10 +489,6 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
         int nstart = 0;
         int scrub = 1;
         int tiebarcount = 1;
-        int pitchcount = 0;
-        int pitchlist[20];
-        int pitchlen[20];
-        int pitchstart[20];
 
         oknbar = FALSE;
 
@@ -506,11 +504,20 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
           }
 
         /* nf_noplay is set when a note has already been played, because of
-        a previous tie, which might have been in a previous bar. */
+        a previous tie, which might have been in a previous bar, or because of
+        a tremolo. */
 
         if ((noteson || main_midifornotesoff) && moff >= midi_bar_moff &&
             note->spitch != 0 && (note->flags & nf_noplay) == 0)
           {
+          int tremolo_multiplier = 0;
+          int pitchlist[20];
+          int pitchlen[20];
+          int pitchstart[20];
+          int pitchcount = 0;
+          int alternate_pitchcount = 0;
+          BOOL alternating;
+
           /* Get a list of pitches in a chord, and leave the general
           pointer p at the final note. */
 
@@ -528,6 +535,50 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
 
           nstart += length;
 
+          /* Skip over a beam break; this is necessary for the [tremolo] check
+          below - and [tremolo] always inserts a beam break. */
+
+          if (note->type == b_beambreak) note = (b_notestr *)note->next;
+
+          /* If a note or chord without a tremolo ornament is followed by
+          [tremolo], that in turn must be followed by another of the same
+          length. If the length is supported, we can then set up the tremolo
+          data, fudging it like a scrub. Start by checking for a supported
+          length. */
+
+          switch(length)
+            {
+            case len_minim:           tremolo_multiplier = 8; break;
+            case (len_minim*3)/2:     tremolo_multiplier = 10; break;
+            case len_crotchet:        tremolo_multiplier = 4; break;
+            case (len_crotchet*3)/2:  tremolo_multiplier = 6; break;
+            case len_quaver:          tremolo_multiplier = 2; break;
+            case (len_quaver*3)/2:    tremolo_multiplier = 3; break;
+            }
+
+          if (note->type == b_tremolo && tremolo_multiplier != 0 &&
+              note->next != NULL && note->next->type == b_note &&
+              (int)((b_notestr *)note->next)->length == length &&
+              scrubtremolo < 0)
+            {
+            for (int i = 0; i < pitchcount; i++) pitchlen[i] *= 2;
+            scrub = ((b_tremolostr *)note)->count * tremolo_multiplier;
+            note = (b_notestr *)note->next;
+
+            /* Add the second note(s) to the pitchlist. */
+
+            do
+              {
+              int i = pitchcount + alternate_pitchcount++;
+              note->flags |= nf_noplay;
+              pitchlist[i] = note->abspitch;
+              pitchlen[i] = 2 * length;
+              pitchstart[i] = nstart - length;
+              note = (b_notestr *)note->next;
+              }
+            while (note->type == b_chord);
+            }
+
           /* If the note is followed by a tie, find the next note or chord on
           the stave. If any of its notes have the same pitch as any of those in
           the list, extend their playing times. If there are any new notes, add
@@ -536,7 +587,7 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
           of this bar. Set the noplay flag in the next notes, to stop them
           playing again later. Continue for multiple ties. */
 
-          while (note->type == b_tie)
+          else while (note->type == b_tie)
             {
             int nlength;
             note = misc_nextnote(note);
@@ -580,23 +631,40 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
 
           /* Handle some common scrubbing */
 
-          if (tremolo > 0 && !thisnotetied)
+          if (scrubtremolo > 0 && !thisnotetied)
             {
-            int ttype = (tremolo == or_trem1)? 1 : 2;
+            int ttype = (scrubtremolo == or_trem1)? 1 :
+                        (scrubtremolo == or_trem2)? 2 : 4;
             switch (length)
               {
-              case len_crotchet:       scrub = 2*ttype; break;
-              case (len_crotchet*3)/2: scrub = 3*ttype; break;
-              case len_minim:          scrub = 4*ttype; break;
-              case (len_minim*3)/2:    scrub = 6*ttype; break;
+              case len_crotchet:        scrub = 2*ttype; break;
+              case (len_crotchet*3)/2:  scrub = 3*ttype; break;
+              case len_minim:           scrub = 4*ttype; break;
+              case (len_minim*3)/2:     scrub = 6*ttype; break;
+              case len_semibreve:       scrub = 8*ttype; break;
+              case (len_semibreve*3)/2: scrub = 10*ttype; break;
               }
             }
 
-          /* The value of "scrub" is 1 for ordinary, non-tremolo notes. */
+          /* The value of "scrub" is 1 for ordinary notes. */
+
+          alternating = (scrub > 1 && scrubtremolo < 0);
 
           for (int scrubcount = 0; scrubcount < scrub; scrubcount++)
             {
-            int pc = pitchcount;
+            int pc;
+            int pc_offset;
+
+            if (alternating && (scrubcount & 1) != 0)
+              {
+              pc = alternate_pitchcount;
+              pc_offset = pitchcount;
+              }
+            else
+              {
+              pc = pitchcount;
+              pc_offset = 0;
+              }
 
             /* For each required pitch, set up the events to make a sound.
             The lengths may be different because of tied/non-tied notes in
@@ -609,10 +677,11 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
 
             while (--pc >= 0)
               {
+              int i = pc + pc_offset;
               int pitch = (midi_stave_pitch != 0)? midi_stave_pitch :
-                pitchlist[pc]/2 + 12 + miditranspose;
-              int start = moff - midi_bar_moff + pitchstart[pc] +
-                scrubcount * (pitchlen[pc]/scrub);
+                pitchlist[i]/2 + 12 + miditranspose;
+              int start = moff - midi_bar_moff + pitchstart[i] +
+                scrubcount * (pitchlen[i]/scrub);
 
               if (pitch < 0 || pitch > 127)
                 {
@@ -651,7 +720,7 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
         moff += length;
         }
 
-      tremolo = -1;
+      scrubtremolo = -1;
       break;
       }  /* End switch on bar item */
     }    /* End of bar scan */
@@ -752,6 +821,12 @@ int32_t mpc;
 
 TRACE("midi_write() movement %d\n", midi_movement);
 
+if (midi_movement < 1 || (usint)midi_movement > movement_count)
+  {
+  error(ERR189, midi_movement);
+  return;
+  }
+
 midi_movt = movements[midi_movement - 1];
 if (midi_movt->barcount < 1)
   {
