@@ -2,17 +2,17 @@
 *              Main header for PMW               *
 *************************************************/
 
-/* Copyright Philip Hazel 2023 */
+/* Copyright Philip Hazel 2025 */
 
 /* PMW rewrite project started: December 2020 */
 /* This file created: December 2020 */
-/* This file last modified: December 2023 */
+/* This file last modified: June 2025 */
 
 /* This file is included by all the other sources except rdargs.c. */
 
-#define PMW_VERSION "5.22"
-#define PMW_DATE    "16-October-2023"
-#define COPYRIGHT   "Copyright (c) Philip Hazel 2023"
+#define PMW_VERSION "5.32-DEV"
+#define PMW_DATE    "05-April-2025"
+#define COPYRIGHT   "Copyright (c) Philip Hazel 2025"
 
 /* Standard C headers */
 
@@ -69,6 +69,7 @@
 #define MAX_MACRODEPTH                10  /* Depth of macro expansion nesting */
 #define MAX_POSTABLESIZE             400  /* Max number of entries */
 #define MAX_REPEATSTYLE                4  /* Max repeat style */
+#define MAX_REPCOUNT                1000  /* Just to catch crazies */
 #define MAX_STAVE                     63  /* Highest stave number */
 #define MAX_UNICODE          0x0010ffffu  /* Largest Unicode code point */
 #define MAX_UTRANSLATE              1000  /* Max Unicode translations for a font */
@@ -96,16 +97,24 @@
 #define UUSIZE                       100  /* For list of unsupported/invalid Unicode codes */
 #define WORDBUFFER_SIZE               64
 
-/* Major program states */
-
-enum { STATE_INIT, STATE_READ, STATE_PAGINATE, STATE_WRITE, STATE_ENDING };
-
 /* Characters whose code point is less than LOWCHARLIMIT can be used directly
 in the "doubled-up" font encoding for standardly-encoded fonts. Some other
 Unicode values are translated when a string is read into code points just above
 this limit, which is no longer Unicode encoding. FONTWIDTHS_SIZE must be large
 enough to accommodate these extra characters. */
 
+/* Major program states */
+
+enum { STATE_INIT, STATE_READ, STATE_PAGINATE, STATE_WRITE, STATE_ENDING };
+
+/* Bits in the main_testing variable. For PS output, any non-zero value runs in
+testing mode. Only for PDF are there separate actions. */
+
+#define mtest_version     0x01u   /* Suppress version and date info */
+#define mtest_barids      0x02u   /* Insert bar and page idents (PDF) */
+#define mtest_omitfont    0x04u   /* Omit font programs (PDF) */
+#define mtest_forcered    0x08u   /* For red output (PDF) */
+
 /* A maximum is set for transposition - 5 octaves should be ample - and a
 conventional value for "no transposition". */
 
@@ -180,8 +189,10 @@ vector as text sizes specified by the user and the fixed sizes. */
 
 #define ff_stdencoding  0x00000001u    /* std encoding */
 #define ff_fixedpitch   0x00000002u    /* fixed pitch */
-#define ff_hasfi        0x00000004u    /* has "fi" */
-#define ff_include      0x00000008u    /* to be included in output */
+#define ff_include      0x00000004u    /* to be included in output */
+#define ff_used         0x00000008u    /* font is used (for PDF output) */
+#define ff_usedlower    0x00000010u    /* lower half used (for std enc) */
+#define ff_usedupper    0x00000020u    /* upper half used (for std enc) */
 
 /* Identifiers for each type of font. Any changes in this list must be kept in
 step with the list of font ids which is kept in Font_IdStrings. The
@@ -257,7 +268,7 @@ typedef uint8_t CBOOL;
 #define Ustrcpy(s,t)         strcpy(CS(s),CCS(t))
 #define Ustrlen(s)           (size_t)strlen(CCS(s))
 #define Ustrncmp(s,t,n)      strncmp(CCS(s),CCS(t),n)
-#define Ustrncmpic(s,t,n)    strncmpic(CCS(s),CCS(t),n)
+#define Ustrncmpic(s,t,n)    strncmpic(CUS(s),CUS(t),n)
 #define Ustrncpy(s,t,n)      strncpy(CS(s),CS(t),n)
 #define Ustrrchr(s,n)        US strrchr(CCS(s),n)
 #define Ustrstr(s, t)        US strstr(CS(s),CS(t))
@@ -265,7 +276,7 @@ typedef uint8_t CBOOL;
 
 /* Miscellaneous definitions. */
 
-#define ENDFILE            0xffffffffu       /* Too large to be a character */
+#define ENDFILE            (-1)
 #define FIXED_UNSET        ((int32_t)(~0u))
 #define KEY_UNSET          0xffffffffu
 #define SIZE_UNSET         ((size_t)(~0uL))
@@ -273,6 +284,8 @@ typedef uint8_t CBOOL;
 /* Functional and shorthand macros */
 
 #define sfb string_format_barnumber
+#define sfd string_format_double
+#define SFD string_format_multiple_double
 #define sff string_format_fixed
 #define SFF string_format_multiple_fixed
 #define sfn string_format_notelength
@@ -324,6 +337,9 @@ typedef uint8_t CBOOL;
 
 #define mac_emptybar(a) (a->next->type == b_barline)
 
+#define poutx(x)  ((x) + print_xmargin)
+#define pouty(y)  (pout_ymax - (y))
+
 /* Tracing and debugging. D_any is set for all -d instances at the start of
 debug decoding. */
 
@@ -371,7 +387,9 @@ enum error_number {
   ERR140,ERR141,ERR142,ERR143,ERR144,ERR145,ERR146,ERR147,ERR148,ERR149,
   ERR150,ERR151,ERR152,ERR153,ERR154,ERR155,ERR156,ERR157,ERR158,ERR159,
   ERR160,ERR161,ERR162,ERR163,ERR164,ERR165,ERR166,ERR167,ERR168,ERR169,
-  ERR170,ERR171,ERR172,ERR173,ERR174,ERR175,ERR176,ERR177,ERR178
+  ERR170,ERR171,ERR172,ERR173,ERR174,ERR175,ERR176,ERR177,ERR178,ERR179,
+  ERR180,ERR181,ERR182,ERR183,ERR184,ERR185,ERR186,ERR187,ERR188,ERR189,
+  ERR190
 };
 
 /* Types of input file */
