@@ -2,9 +2,9 @@
 *        PMW native header reading functions     *
 *************************************************/
 
-/* Copyright Philip Hazel 2021 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: December 2020 */
-/* This file last modified: December 2023 */
+/* This file last modified: April 2025 */
 
 #include "pmw.h"
 
@@ -455,6 +455,33 @@ else x[ac_no] = x[ac_sh] - 200;
 
 
 
+/*************************************************
+*           B2PF callback function               *
+*************************************************/
+
+/* Check a proposed ligature for availability in the relevant font.
+
+Arguments:
+  codepoint   a Unicode code point
+  data        opaque data is the font number
+
+Returns:      0 this ligature is not acceptable
+              1 this ligature is acceptable
+*/
+
+#if defined SUPPORT_B2PF && SUPPORT_B2PF != 0 && defined B2PF_CALLBACK_LIGATURE
+static int
+callback(uint32_t codepoint, void *data)
+{
+fontstr *fs = &(font_list[font_table[(intptr_t)data]]);
+uint32_t c = font_utranslate(codepoint, fs);
+if (c == 0xffffffffu) c = codepoint;   /* No Unicode translation */
+return (c < 512)? 1 : 0;
+}
+#endif
+
+
+
 /*************************************************
 *       Set up font for B2PF processing          *
 *************************************************/
@@ -519,6 +546,14 @@ rc = b2pf_context_create((const char *)read_stringbuffer,
   (const char *)font_data_extra, 0, font_b2pf_contexts + fontid,
   NULL, NULL, NULL, &ln);
 
+/* Set up the ligature-checking callback, if we are on a suitable release. */
+
+#ifdef B2PF_CALLBACK_LIGATURE
+if (rc == B2PF_SUCCESS)
+  rc = b2pf_context_set_callback(font_b2pf_contexts[fontid],
+    B2PF_CALLBACK_LIGATURE, callback, (void *)((intptr_t)fontid));
+#endif
+
 /* If there are more strings in quotes, add them as extra context rules. */
 
 if (rc == B2PF_SUCCESS)
@@ -543,6 +578,8 @@ if (rc != B2PF_SUCCESS)
   error(ERR79, buffer);  /* Hard */
   }
 
+font_call_b2pf = TRUE;  /* Tell string_check to use B2PF */
+
 #endif  /* SUPPORT_B2PF */
 }
 
@@ -796,11 +833,24 @@ if (movement_count == 1) stave_use_draw = x;
 *                 EPS                            *
 *************************************************/
 
+/* This is synonym for "output eps", which existed before PDF output was
+implemented. It's small enough not to bother with trying to combine it with the
+"output" directive. */
+
 static void
 eps(void)
 {
-if (movement_count == 1) print_imposition = pc_EPS;
-  else error(ERR40, dir->name);
+if (movement_count == 1)
+  {
+  if (!PDF || !PDFforced)
+    {
+    print_imposition = pc_EPS;
+    PDF = FALSE;
+    PSforced = TRUE;
+    }
+  else error(ERR188, "eps", "-pdf");
+  }
+else error(ERR40, dir->name);
 }
 
 
@@ -1181,7 +1231,7 @@ for (i = 0;; i++)
     }
   else
     {
-    uint32_t peek_c = main_readbuffer[read_i];
+    int32_t peek_c = main_readbuffer[read_i];
     ac = (read_c == '#')? ac_sh : ac_fl;
     if (peek_c == read_c)
       {
@@ -1583,6 +1633,53 @@ else
 
 
 
+/*************************************************
+*                Output                          *
+*************************************************/
+
+static void
+output(void)
+{
+if (movement_count != 1) { error(ERR40, dir->name); return; }
+read_nextword();
+read_sigc();
+
+if (Ustrcmp(read_wordbuffer, "eps") == 0)
+  {
+  if (!PDF || !PDFforced)
+    {
+    print_imposition = pc_EPS;
+    PDF = FALSE;
+    EPSforced = TRUE;
+    }
+  else error(ERR188, "output eps", "PDF");
+  }
+else if (Ustrcmp(read_wordbuffer, "pdf") == 0)
+  {
+  if (!PSforced && !EPSforced)
+    {
+    PDF = PDFforced = TRUE;
+    print_imposition = pc_normal;
+    }
+  else error(ERR188, "output pdf", "PostScript");
+  }
+else if (Ustrcmp(read_wordbuffer, "ps") == 0 ||
+         Ustrcmp(read_wordbuffer, "postscript") == 0)
+  {
+  if (!PDFforced && !EPSforced)
+    {
+    PDF = FALSE;
+    PSforced = TRUE;
+    print_imposition = pc_normal;
+    }
+  else error(ERR188, "output ps", PDFforced? "PDF" : "eps");
+  }
+else error(ERR8, "\"eps\", \"pdf\", or \"ps\"");
+}
+
+
+
+
 /*************************************************
 *               Page                             *
 *************************************************/
@@ -2433,6 +2530,7 @@ static dirstr headlist[] = {
   { "oldrestlevel",     warning,        0, 0 },
   { "oldstemlength",    warning,        0, 0 },
   { "oldstretchrule",   warning,        0, 0 },
+  { "output",           output,         0, 0 },
   { "overlaydepth",     movt_int,       oo(movtstr,overlaydepth), int_f },
   { "overlaysize",      movt_fontsize,  oo(fontsizestr,fontsize_text)+ff_offset_olay*sizeof(fontinststr), TRUE },
   { "page",             page,           0, 0 },
