@@ -2,9 +2,9 @@
 *  PMW native stave directive reading functions  *
 *************************************************/
 
-/* Copyright Philip Hazel 2023 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: February 2021 */
-/* This file last modified: December 2023 */
+/* This file last modified: April 2025 */
 
 #include "pmw.h"
 
@@ -51,18 +51,18 @@ static int clef_octave[] = {
     0,   /* baritone */
     0,   /* bass */
     0,   /* cbaritone */
-  -12,   /* contrabass */
+  -24,   /* contrabass */
     0,   /* deepbass */
     0,   /* hclef */
     0,   /* mezzo */
     0,   /* none */
-   12,   /* soprabass */
+   24,   /* soprabass */
     0,   /* soprano */
     0,   /* tenor */
     0,   /* treble */
-   12,   /* trebledescant */
-  -12,   /* trebletenor */
-  -12    /* trebletenorB */
+   24,   /* trebledescant */
+  -24,   /* trebletenor */
+  -24    /* trebletenorB */
 };
 
 
@@ -972,6 +972,10 @@ while(read_c == '\"' ||
 
   read_sigc();
   }
+
+/* Give an error if no strings or drawings were specified. */
+
+if (p == NULL) error_skip(ERR8, ']', "string or \"draw\"");
 }
 
 
