@@ -2,9 +2,9 @@
 *         PMW general reading functions          *
 *************************************************/
 
-/* Copyright Philip Hazel 2021 */
+/* Copyright Philip Hazel 2024 */
 /* This file created: December 2020 */
-/* This file last modified: May 2023 */
+/* This file last modified: December 2024 */
 
 #include "pmw.h"
 
@@ -230,7 +230,7 @@ char *sss = (char *)ss;
 if (map != NULL) *map = 0;
 if (slp != NULL) *slp = NULL;
 
-while (isdigit(*sss))
+while (isdigit((unsigned char)*sss))
   {
   long int s = strtol(sss, &sss, 0);
   long int t = s;
@@ -268,11 +268,20 @@ return 0;
 *           Extend input buffers                 *
 *************************************************/
 
+/* When the main readbuffer is too small it means the input is crazy. Rather
+than showing thousands of characters in the error message, truncate it. Setting
+read_i and macro_i to zero (to cover both the cases of main input or expanded
+input) stops the output of a current pointer. */
+
 void
 read_extend_buffers(void)
 {
 if (main_readbuffer_size >= MAIN_READBUFFER_SIZELIMIT)
+  {
+  Ustrcpy(main_readbuffer+50, "..... etc ......\n");
+  read_i = macro_in = 0;
   error(ERR5, "input", MAIN_READBUFFER_SIZELIMIT);  /* Hard */
+  }
 main_readbuffer_size += MAIN_READBUFFER_CHUNKSIZE;
 main_readbuffer = realloc(main_readbuffer, main_readbuffer_size);
 main_readbuffer_previous = realloc(main_readbuffer_previous,
@@ -340,6 +349,9 @@ Returns:   TRUE if line read, FALSE at EOF
 BOOL
 read_physical_line(size_t i)
 {
+BOOL binfound = FALSE;
+size_t binoffset;
+
 if (i == 0 && main_readbuffer[0] != 0 && main_readbuffer[0] != '\n')
   {
   uschar *temp = main_readbuffer_previous;
@@ -353,11 +365,16 @@ for (;;)
   {
   int ch = fgetc(read_filehandle);
 
-  /* Binary zeros are not supported. Give an error and ignore. */
+  /* Binary zeros are not supported. Remember where the first one was. */
 
   if (ch == 0)
     {
-    error(ERR2);
+    if (!binfound)
+      {
+      binoffset = i;
+      if (binoffset == 0) binoffset++;
+      binfound = TRUE;
+      }
     continue;
     }
 
@@ -386,6 +403,15 @@ for (;;)
   main_readbuffer[i++] = ch;
   }
 
+/* Give an error if any binary zeros were found - can't do earlier as we need
+the complete line for reflection. */
+
+if (binfound)
+  {
+  read_i = binoffset;
+  error(ERR2);
+  }
+
 main_readlength = i;
 return TRUE;
 }
