@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: June 2021 */
-/* This file last modified: September 2021 */
+/* This file last modified: June 2025 */
 
 #include "pmw.h"
 
@@ -185,13 +185,13 @@ if (!skip)
   {
   if ((h0->flags & hp_cresc) == 0)
     {
-    ps_line(x0, y0 + dwidth, x1, y1 + y1hole, thickness, 0);
-    ps_line(x0, y0 - dwidth, x1, y1 - y1hole, thickness, 0);
+    ofi_line(x0, y0 + dwidth, x1, y1 + y1hole, thickness, 0);
+    ofi_line(x0, y0 - dwidth, x1, y1 - y1hole, thickness, 0);
     }
   else
     {
-    ps_line(x0, y0 + y0hole, x1, y1 + cwidth, thickness, 0);
-    ps_line(x0, y0 - y0hole, x1, y1 - cwidth, thickness, 0);
+    ofi_line(x0, y0 + y0hole, x1, y1 + cwidth, thickness, 0);
+    ofi_line(x0, y0 - y0hole, x1, y1 - cwidth, thickness, 0);
     }
   }
 
@@ -384,13 +384,20 @@ switch (p->type)
   /* Deal with inter-note tremolo; remember data for use after next note. */
 
   case b_tremolo:
-  if (n_pitch != 0)
+  {
+  b_notestr *pp = misc_nextnote(p);
+  if (pp == NULL || pp->spitch == 0)
+    error(ERR190, "no following note");
+  else if (n_pitch == 0)
+    error(ERR190, "no preceding note");
+  else
     {
     out_tremolo = (b_tremolostr *)p;
     out_tremupflag = n_upflag;
     out_tremx = n_x;
     out_tremy = misc_ybound(n_notetype < minim || !n_upflag, NULL, FALSE, FALSE);
     }
+  }
   break;
 
   /* Deal with plets */
@@ -458,13 +465,14 @@ switch (p->type)
       ss = string_pmw(s, curmovt->fonttype_triplet);
       width = string_width(ss, &fdata, NULL)/2;
 
-      /* If beamed and no line, compute text position according to the beam;
-      otherwise compute it as central in the line */
+      /* If beamed and no line, compute text position according to the beam.
+      Left-to-right needs a fudge for up-beams, right-to-left on down beams.
+      Otherwise compute it as central in the line. */
 
       if (beam_seq == 1 && above == beam_upflag && omitline)
         {
         sx = (out_plet_x + n_x)/2 + out_plet->x;
-        if (beam_upflag) sx += (51*out_stavemagn)/10;
+        if (beam_upflag != main_righttoleft) sx += (51*out_stavemagn)/10;
         }
       else sx = mid + (omitline? out_plet->x : 0);
 
@@ -493,14 +501,14 @@ switch (p->type)
         y[0] = yyl + ly;
         y[1] = yyl;
         y[2] = yyl + mac_muldiv(slope, x[2] - x0, 1000);
-        ps_lines(x, y, 3, curmovt->tripletlinewidth);
+        ofi_lines(x, y, 3, curmovt->tripletlinewidth);
 
         x[0] = mid + 1500 + width;
         x[1] = x[2] = x1;
         y[0] = yyr - mac_muldiv(slope, x1 - x[0], 1000);
         y[1] = yyr;
         y[2] = yyr + ry;
-        ps_lines(x, y, 3, curmovt->tripletlinewidth);
+        ofi_lines(x, y, 3, curmovt->tripletlinewidth);
         }
       }
     }
@@ -535,7 +543,7 @@ switch (p->type)
       if (!out_startlinebar && out_moff == 0 &&
           MFLAG(mf_keydoublebar) && !out_repeatonbarline &&
           curbarnumber != curmovt->startbracketbar)
-        ps_barline(out_lastbarlinex, out_ystave, out_ybarend, bar_double,
+        ofi_barline(out_lastbarlinex, out_ystave, out_ybarend, bar_double,
           ((curmovt->barlinesize > 0)? curmovt->barlinesize : out_stavemagn));
       out_writekey(x, out_ystave - out_Yadjustment, bar_cont->clef, k->key);
       }
@@ -558,7 +566,7 @@ switch (p->type)
   /* Dotted bar line in mid-bar */
 
   case b_dotbar:
-  ps_barline(out_barx+out_findXoffset(out_moff+posx_dotbar)+out_Xadjustment,
+  ofi_barline(out_barx+out_findXoffset(out_moff+posx_dotbar)+out_Xadjustment,
     out_ystave, out_ybarend, bar_dotted, out_barlinemagn);
   out_Xadjustment = out_Yadjustment = 0;
   break;
